import CardRechargeRepository from '../Repositories/CardRechargeRepository.js'
import BalanceRepository from '../Repositories/BalanceRepository.js'
import EvoluiSaldoRepository from '../Repositories/EvoluiSaldoRepository.js'
import EvoluiSaldoController from './EvoluiSaldoController.js'
import RechargeController from './RechargeController.js'
import BaseInfoController from './BaseInfoController.js'
import EmailController from './EmailController.js'
import { doPaymentThroughAPI, confirmPayment, cancelPayment } from './PaymentController.js'
import moment from 'moment'
import { commit, rollback } from './../Connection/QueriesExecutor.js'
import { log } from './LogController.js'

const getCardRechargeDetails = async function ({ consDetails }) {
    const consumerValidation = await CardRechargeRepository.checkConsumerDetails(consDetails)
    if (!consumerValidation[0] || (consumerValidation[0] && (!consumerValidation[0].NMRESPCONS || !consumerValidation[0].DSEMAILCONS))) {
        return {
            error: true,
            message: 'Dados do consumidor não encontrados'
        }
    }

    return await CardRechargeRepository.getCardRechargeDetails(consDetails)
}

const doCardRecharge = async function ({ totalValue, flag, cardExpiration, cardNumber, cardToken, cardCVV, cardHolder, rechargeValues, consDetails, mode, tipoRece = null, finalNumbers }) {
    const { NMRESPCONS, DSEMAILCONS } = consDetails
    let nsu = ''
    let nit = ''
    let NRTRANSACAO = ''
    const connectionKey = consDetails.CDCLIENTE + consDetails.CDCONSUMIDOR

    try {
        NRTRANSACAO = await newCode('TRANSACAO', 12)
    
        // insert transaction
        const paramsTransaction = {
            NRTRANSACAO,
            DSTRANSACAO: 'Transação iniciada.',
            CDCLIENTE: consDetails.CDCLIENTE,
            CDCONSUMIDOR: consDetails.CDCONSUMIDOR,
            VRMOVEXTCONS: totalValue
        }
    
        await CardRechargeRepository.insertTransaction(paramsTransaction)
    
        // transaction consumer - todo: check if a loop through rechargeValues is needed <<<
        const paramsTransactionConsumer = {
            NRTRANSACAO,
            CDCLIENTE: consDetails.CDCLIENTE,
            CDCONSUMIDOR: consDetails.CDCONSUMIDOR
        }
    
        await CardRechargeRepository.insertTransactionConsumer(paramsTransactionConsumer)
    
        // call payment API
        let response = null

        if (cardNumber) {
            response = await doPaymentThroughAPI({ totalValue, flag, cardExpiration, cardNumber, cardCVV, cardHolder, mode, NRTRANSACAO })
        } else {
            response = await doPaymentThroughAPI({ totalValue, flag, cardExpiration, cardToken, cardCVV, cardHolder, mode, NRTRANSACAO })
        }
    
        // if the status is 500, return error, else, gets nsu and nit
        if (!response || response.status === 500) {
            return {
                error: true,
                message: 'Erro ao realizar pagamento.'
            }
        }
        
        nsu = response?.data?.result?.nsu || ''
        nit = response?.data?.result?.nit || ''

        console.log('NRTRANSACAO', NRTRANSACAO)
        console.log('nsu', nsu)

        //  update transaction
        let paramsTransactionUpdate = {
            NRTRANSACAO,
            DSTRANSACAO: 'Transação digitada aprovada. Inserindo créditos.',
            NRNIT: nsu
        }
    
        await CardRechargeRepository.updateTransaction(paramsTransactionUpdate)
    
        // update balance and insert into EXTRATOCONS
        await RechargeController.updateBalance(rechargeValues, consDetails, tipoRece, NRTRANSACAO, nsu, connectionKey)

        // confirm payment and commit/rollback
        const confirmationResponse = await confirmPayment(nit)

        if (!confirmationResponse || confirmationResponse.status === 500) {
            await rollback({ connectionKey })
            return {
                error: true,
                message: 'Erro ao realizar pagamento.'
            }
        } else {
            await commit({ connectionKey })
        }

        // send confirmation email
        let cardCode = 'XXXX XXXX XXXX '
        cardCode += finalNumbers ? finalNumbers : cardNumber.substring(12)
        const transactionValue = parseFloat(totalValue).toLocaleString('pt-br', { style: 'currency', currency: 'BRL' })
        const transactionDate = moment().format('DD/MM/YYYY HH:mm:ss')
        // loops through rechargeValues to get students names - NMCONSUMIDOR
        const studentsNames = rechargeValues.map((family) => family.NMCONSUMIDOR).join(', ')

        const emailData = {
            name: NMRESPCONS,
            studentsNames,
            cardCode,
            transactionValue,
            transactionDate
        }

        const emailResult = await EmailController.sendEmail({ DSEMAILCONS, subject: 'Comprovante de Recarga do Cartão', consDetails, htmlFunction: 'getPaymentConfirmationEmail', functionParams: emailData})

        if (emailResult.accepted?.length > 0) {
            return {
                error: false,
                message: 'Recarga realizada com sucesso. Comprovante enviado por e-mail.'
            }
        } else {
            return {
                error: false,
                message: 'Recarga realizada com sucesso. Não foi possível enviar o comprovante por e-mail.'
            }
        }
    } catch (error) {
        log({ type: 'error', message: `Ocorreu um erro em doCardRecharge\nNSU: ${nsu}\nNRTRANSACAO: ${NRTRANSACAO}\nNIT: ${nit}\n${error.message}` })

        await rollback({ connectionKey })
        await cancelPayment(nit)

        const paramsTransactionUpdate = {
            NRTRANSACAO,
            DSTRANSACAO: 'Erro na recarga do cartão: ' + error.message,
            NRNIT: nsu ? nsu : null
        }

        await CardRechargeRepository.updateTransaction(paramsTransactionUpdate)

        return {
            error: true,
            message: 'Erro ao realizar recarga.'
        }
    }
}

const newCode = async function (counterName, size) {
    const baseInfo = await BaseInfoController.getBaseInfo()
    const NRORG = baseInfo[0].NRORG

    const paramsNewCode = {
        NRORGNOVOCODIGO: NRORG,
        NOMETABELA: counterName
    }

    let newCode = await CardRechargeRepository.getNewCode(paramsNewCode)
    if (newCode?.length && size) newCode = newCode.substring(newCode.length - size)
    return newCode
}

export default { getCardRechargeDetails, doCardRecharge }
