import RechargeRepository from './../Repositories/RechargeRepository.js'
import EvoluiSaldoRepository from '../Repositories/EvoluiSaldoRepository.js'
import EvoluiSaldoController from './EvoluiSaldoController.js'
import BalanceRepository from '../Repositories/BalanceRepository.js'
import { consultPayment } from './PaymentController.js'

import { commit, rollback } from './../Connection/QueriesExecutor.js'

const updateBalance = async (rechargeValues, consDetails, tipoRece = null, NRTRANSACAO, nsu = null, connectionKey = null) => {
    const { CDCLIENTE, CDFILIAL, CDTIPOCONS } = consDetails

    // starts a loop to insert recharge values
    for (let i = 0; i < rechargeValues.length; i++) {            
        const family = rechargeValues[i]

        const isFirstInsertion = await RechargeRepository.isFirstInsertion({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR, CDFAMILISALD: family.CDFAMILISALD }, false, connectionKey)
        const currentBalance = await RechargeRepository.getCurrentBalance({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR, CDFAMILISALD: family.CDFAMILISALD }, false, connectionKey)
        
        const valueToRecharge = family.VRSALDCONFAM

        // check if the consumer has a balance
        const balance = await BalanceRepository.checkBalance({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR, CDFAMILISALD: family.CDFAMILISALD }, false, connectionKey)
        
        let VRSALDCONEXT = valueToRecharge

        // if it's not the first insertion into EXTRATOCONS, we have to add the previous balance to the new insertion
        if (isFirstInsertion.length > 0) {
            if (currentBalance.length > 0) {
                VRSALDCONEXT = Math.round((VRSALDCONEXT + parseFloat(currentBalance[0].VRSALDCONEXT)) * 100) / 100 // new balance
            }
        }

        // todo - check CDTIPORECE and CDFILIAL to insert into EXTRATOCONS <<<
        const NRSEQMOVEXT = await RechargeRepository.getNRSEQMOVEXT({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR }, false, connectionKey)

        await RechargeRepository.insertExtratoCons({
            CDCLIENTE,
            CDCONSUMIDOR: family.CDCONSUMIDOR,
            CDFAMILISALD: family.CDFAMILISALD,
            DSOPEEXTCONS: 'Credito NoCash (Cartao)',
            VRMOVEXTCONS: valueToRecharge,
            IDTPMOVEXT: 'C',
            NRSEQMOVEXT: NRSEQMOVEXT[0].NRSEQMOVEXT,
            VRSALDCONEXT,
            CDTIPORECE: tipoRece,
            CDFILIAL
        }, false, connectionKey)  

        // if there is no record on SALDOCONS, we have to create one with current valueToRecharge
        if (balance.length === 0) {
            // Without any balance, the balance can't be digivolved, so we must insert additional balance
            await BalanceRepository.createBalance({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR, CDFAMILISALD: family.CDFAMILISALD, VRSALDCONFAM: VRSALDCONEXT }, false, connectionKey)
        } else {
            // if there is balance already, we have to update it with the new value
            await BalanceRepository.updateBalance({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR, CDFAMILISALD: family.CDFAMILISALD, VRSALDCONFAM: VRSALDCONEXT }, false, connectionKey)
        }

        // bonus recharge
        const bonusRecharge = await BalanceRepository.getBonusRecharge({ CDFILIAL, CDTIPOCONS, CDTIPORECEORIGEM: tipoRece }, false, connectionKey)
        let vrPrevConExt = 0
        if (bonusRecharge.length > 0) {
            if (currentBalance.length > 0) {
                vrPrevConExt = parseFloat(currentBalance[0].VRSALDCONEXT)
            } else {
                vrPrevConExt = 0
            }

            if (valueToRecharge >= bonusRecharge[0].VRCARGACREDITO && vrPrevConExt < bonusRecharge[0].VRSALDCONSBONUS) {
                VRSALDCONEXT = Math.round((VRSALDCONEXT + parseFloat(bonusRecharge[0].VRBONUSCREDITO)) * 100) / 100

                await RechargeRepository.insertExtratoCons({
                    CDCLIENTE,
                    CDCONSUMIDOR: family.CDCONSUMIDOR,
                    CDFAMILISALD: family.CDFAMILISALD,
                    DSOPEEXTCONS: 'Bonus NoCash (Cartao)',
                    VRMOVEXTCONS: bonusRecharge[0].VRBONUSCREDITO,
                    IDTPMOVEXT: 'C',
                    NRSEQMOVEXT: NRSEQMOVEXT[0].NRSEQMOVEXT,
                    VRSALDCONEXT,
                    CDTIPORECE: tipoRece,
                    CDFILIAL
                }, false, connectionKey)

                await BalanceRepository.updateBalance({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR, CDFAMILISALD: family.CDFAMILISALD, VRSALDCONEXT }, false, connectionKey)
            }
        }

        // recalculate balance process
        const evoluiSaldoDate = await EvoluiSaldoRepository.getTransactionDateF({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR }, false, connectionKey)
        const NRORG = await EvoluiSaldoRepository.getConsumerNrorg({ CDCLIENTE, CDCONSUMIDOR: family.CDCONSUMIDOR }, false, connectionKey)

        const conditions = {
            CDCLIENTE,
            CDCONSUMIDOR: family.CDCONSUMIDOR,
            DTMOVEXTCONS: evoluiSaldoDate[0].DTMOVEXTCONS,
            NRORG: NRORG[0].NRORG,
        }

        await EvoluiSaldoController.EvoluiSaldoFunction(conditions, connectionKey)
    } // ends for

    // update transaction with success message
    const paramsTransactionUpdate = {
        NRTRANSACAO,
        DSTRANSACAO: 'Transação finalizada com sucesso.',
        NRNIT: nsu
    }
    await RechargeRepository.updateTransaction(paramsTransactionUpdate, false, connectionKey)
    console.log('retornando recharge controller')
}

const checkSonsBalances = async ({ consDetails }) => {
    const connectionKey = consDetails.CDCLIENTE + consDetails.CDCONSUMIDOR
    try {
        const { CDCLIENTE, CDCONSUMIDOR } = consDetails
        const checkTempRechargeCons = await RechargeRepository.checkTempRechargeCons({ CDCLIENTE, CDCONSUMIDOR }, false, connectionKey)
    
        for (let i = 0; i < checkTempRechargeCons.length; i++) {
            const recharge = checkTempRechargeCons[i]
            const { NRTRANSACAO } = recharge
            const rechargeValues = JSON.parse(recharge.RECHARGE_VALUES)
            const { CDCLIENTE, CDFILIAL, CDTIPOCONS, TIPORECE, NSU, NIT } = recharge

            // checks if the payment was confirmed with the API
            const apiResponse = consultPayment(NIT)
            if (apiResponse?.data?.result?.status !== 'CONFIRMED') {
                await RechargeRepository.deleteTempRecharge({NRTRANSACAO}, false, connectionKey)
                continue
            }
    
            await updateBalance(rechargeValues, { CDCLIENTE, CDFILIAL, CDTIPOCONS }, TIPORECE, NRTRANSACAO, NSU)
            await RechargeRepository.deleteTempRecharge({NRTRANSACAO}, false, connectionKey)
        }
    
        await commit({ connectionKey })
    } catch (error) {
        await rollback({ connectionKey })
        console.log(error)
    }
}

export default { updateBalance, checkSonsBalances }